import pytest
from hamcrest import assert_that, anything, not_
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_parent_suite
from allure_commons_test.label import has_suite
from allure_commons_test.label import has_sub_suite


@pytest.mark.skip
def test_default_suite(executed_docstring_source):
    """
    >>> def test_default_suite_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_default_suite_example",
                              has_parent_suite(anything()),  # path to testdir
                              has_suite("test_default_suite"),  # created file name
                              not_(has_sub_suite(anything()))
                              )
                )


@pytest.mark.skip
def test_default_class_suite(executed_docstring_source):
    """
    >>> class TestSuiteClass(object):
    ...     def test_default_class_suite_example(self):
    ...         pass

    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_default_class_suite_example",
                              has_parent_suite(anything()),  # path to testdir
                              has_suite("test_default_class_suite"),  # created file name
                              has_sub_suite("TestSuiteClass")
                              )
                )



import pytest
from hamcrest import assert_that
from allure_commons.utils import represent
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_step
from allure_commons_test.result import has_parameter


def params_name(request):
    node_id = request.node.nodeid
    _, name = node_id.rstrip("]").split("[")
    return name


@pytest.mark.parametrize(
    ["args", "kwargs"],
    [
        ([True], {"kwarg": False}),
        ([True], {"kwarg": False}),
        (["hi"], {"kwarg": None}),
        ([None], {"kwarg": 42})
    ]
)
def test_step_parameters(executed_docstring_source, request, args, kwargs):
    """
    >>> import pytest
    >>> import allure

    >>> @allure.step
    ... def step(arg, kwarg=None):
    ...     pass

    >>> @pytest.mark.parametrize(
    ...     ["args", "kwargs"],
    ...     [
    ...         ([True], {"kwarg": False}),
    ...         ([True], {"kwarg": False}),
    ...         (["hi"], {"kwarg": None}),
    ...         ([None], {"kwarg": 42})
    ...     ]
    ... )
    ... def test_args_less_than_placeholders_example(args, kwargs):
    ...     step(*args, **kwargs)
    """

    test_name = "test_args_less_than_placeholders_example[{params_name}]".format(
        params_name=params_name(request))

    assert_that(executed_docstring_source.allure_report,
                has_test_case(test_name,
                              has_step("step",
                                       *([has_parameter("arg", represent(arg)) for arg in args] +
                                         [has_parameter("kwarg", represent(kwarg)) for kwarg in kwargs.values()]
                                         )

                                       )
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains


def test_passed(executed_docstring_source):
    """
    >>> def test_passed_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_passed_example",
                              with_status("passed")
                              )
                )


def test_failed(executed_docstring_source):
    """
    >>> def test_failed_example():
    ...     assert False
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_failed_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("AssertionError"),
                                                 with_trace_contains("def test_failed_example():")
                                                 )
                              )
                )


def test_broken(executed_docstring_source):
    """
    >>> def test_broken_example():
    ...     raise IndentationError()
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_broken_example",
                              with_status("broken"),
                              has_status_details(with_message_contains("IndentationError"),
                                                 with_trace_contains("def test_broken_example():")
                                                 )
                              )
                )


def test_call_pytest_fail(executed_docstring_source):
    """
    >>> import pytest

    >>> def test_call_pytest_fail_example():
    ...     pytest.fail()
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_call_pytest_fail_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("Failed: <Failed instance>"),
                                                 with_trace_contains("def test_call_pytest_fail_example():")
                                                 )
                              )
                )


def test_call_pytest_fail_with_reason(executed_docstring_source):
    """
    >>> import pytest

    >>> def test_call_pytest_fail_with_reason_example():
    ...     pytest.fail("Fail message")
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_call_pytest_fail_with_reason_example",
                              with_status("failed"),
                              has_status_details(with_message_contains("Fail message"),
                                                 with_trace_contains("def test_call_pytest_fail_with_reason_example():")
                                                 )
                              )

                )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import six
import pytest
from itertools import chain, islice
from allure_commons.utils import represent
from allure_commons.utils import format_exception, format_traceback, escape_non_unicode_symbols
from allure_commons.model2 import Status
from allure_commons.model2 import StatusDetails
from allure_commons.types import LabelType


ALLURE_DISPLAY_NAME_MARK = 'allure_display_name'
ALLURE_DESCRIPTION_MARK = 'allure_description'
ALLURE_DESCRIPTION_HTML_MARK = 'allure_description_html'
ALLURE_LABEL_MARK = 'allure_label'
ALLURE_LINK_MARK = 'allure_link'
ALLURE_UNIQUE_LABELS = [
    LabelType.SEVERITY,
    LabelType.FRAMEWORK,
    LabelType.HOST,
    LabelType.SUITE,
    LabelType.PARENT_SUITE,
    LabelType.SUB_SUITE
]


def get_marker_value(item, keyword):
    marker = item.get_closest_marker(keyword)
    return marker.args[0] if marker and marker.args else None


def allure_title(item):
    return get_marker_value(item, ALLURE_DISPLAY_NAME_MARK)


def allure_description(item):
    description = get_marker_value(item, ALLURE_DESCRIPTION_MARK)
    if description:
        return description
    elif hasattr(item, 'function'):
        return item.function.__doc__


def allure_description_html(item):
    return get_marker_value(item, ALLURE_DESCRIPTION_HTML_MARK)


def allure_label(item, label):
    labels = []
    for mark in item.iter_markers(name=ALLURE_LABEL_MARK):
        if mark.kwargs.get("label_type") == label:
            labels.extend(mark.args)
    return labels


def allure_labels(item):
    unique_labels = dict()
    labels = set()
    for mark in item.iter_markers(name=ALLURE_LABEL_MARK):
        label_type = mark.kwargs["label_type"]
        if label_type in ALLURE_UNIQUE_LABELS:
            if label_type not in unique_labels.keys():
                unique_labels[label_type] = mark.args[0]
        else:
            for arg in mark.args:
                labels.add((label_type, arg))
    for k, v in unique_labels.items():
        labels.add((k, v))
    return labels


def allure_links(item):
    for mark in item.iter_markers(name=ALLURE_LINK_MARK):
        yield (mark.kwargs["link_type"], mark.args[0], mark.kwargs["name"])


def pytest_markers(item):
    """Do not consider pytest marks (has args/kwargs) as user tags
    e.g. @pytest.mark.parametrize/skip/skipif/usefixtures etc..."""
    for keyword in item.keywords.keys():
        if keyword.startswith('allure_'):
            continue
        marker = item.get_closest_marker(keyword)
        if marker is None:
            continue
        user_tag_mark = (not marker.args and not marker.kwargs)
        if marker.name == "marker" or user_tag_mark:
            yield mark_to_str(marker)


def mark_to_str(marker):
    args = [represent(arg) for arg in marker.args]
    kwargs = ['{name}={value}'.format(name=key, value=represent(marker.kwargs[key])) for key in marker.kwargs]
    markstr = '{name}'.format(name=marker.name)
    if args or kwargs:
        parameters = ', '.join(args + kwargs)
        markstr = '{}({})'.format(markstr, parameters)
    return markstr


def allure_package(item):
    parts = item.nodeid.split('::')
    path = parts[0].rsplit('.', 1)[0]
    return path.replace('/', '.')


def allure_name(item, parameters):
    name = escape_name(item.name)
    title = allure_title(item)
    return title.format(**parameters) if title else name


def allure_full_name(item):
    parts = item.nodeid.split('::')
    package = allure_package(item)
    clazz = '.{clazz}'.format(clazz=parts[1]) if len(parts) > 2 else ''
    test_with_params = parts[-1]
    test = test_with_params.rsplit("[", 1)[0]
    full_name = '{package}{clazz}#{test}'.format(package=package, clazz=clazz, test=test)
    return escape_name(full_name)


def allure_suite_labels(item):
    head, possibly_clazz, tail = islice(chain(item.nodeid.split('::'), [None], [None]), 3)
    clazz = possibly_clazz if tail else None
    file_name, path = islice(chain(reversed(head.rsplit('/', 1)), [None]), 2)
    module = file_name.split('.')[0]
    package = path.replace('/', '.') if path else None
    pairs = dict(zip([LabelType.PARENT_SUITE, LabelType.SUITE, LabelType.SUB_SUITE], [package, module, clazz]))
    labels = dict(allure_labels(item))
    default_suite_labels = []
    for label, value in pairs.items():
        if label not in labels.keys() and value:
            default_suite_labels.append((label, value))

    return default_suite_labels


def escape_name(name):
    if six.PY2:
        try:
            name.decode('string_escape').encode('unicode_escape')
        except UnicodeDecodeError:
            return name.decode('string_escape').decode('utf-8')
    return name.encode('ascii', 'backslashreplace').decode('unicode_escape')


def get_outcome_status(outcome):
    _, exception, _ = outcome.excinfo or (None, None, None)
    return get_status(exception)


def get_outcome_status_details(outcome):
    exception_type, exception, exception_traceback = outcome.excinfo or (None, None, None)
    return get_status_details(exception_type, exception, exception_traceback)


def get_status(exception):
    if exception:
        if isinstance(exception, AssertionError) or isinstance(exception, pytest.fail.Exception):
            return Status.FAILED
        elif isinstance(exception, pytest.skip.Exception):
            return Status.SKIPPED
        return Status.BROKEN
    else:
        return Status.PASSED


def get_status_details(exception_type, exception, exception_traceback):
    message = escape_non_unicode_symbols(format_exception(exception_type, exception))
    trace = escape_non_unicode_symbols(format_traceback(exception_traceback))
    return StatusDetails(message=message, trace=trace) if message or trace else None


def get_pytest_report_status(pytest_report):
    pytest_statuses = ('failed', 'passed', 'skipped')
    statuses = (Status.FAILED, Status.PASSED, Status.SKIPPED)
    for pytest_status, status in zip(pytest_statuses, statuses):
        if getattr(pytest_report, pytest_status):
            return status



import pytest
from hamcrest import assert_that
from hamcrest import all_of, is_, is_not, empty
from hamcrest import has_property, has_value
from hamcrest import contains_string


@pytest.mark.parametrize("capture", ["sys", "fd", "no"])
def test_capture_stdout(allured_testdir, capture):
    """
    >>> import pytest
    >>> import allure

    >>> @pytest.fixture
    ... def fixture(request):
    ...     print ("Start fixture")
    ...     def finalizer():
    ...         print ("Stop fixture")
    ...     request.addfinalizer(finalizer)

    >>> def test_capture_stdout_example(fixture):
    ...     print ("Start test")
    ...     with allure.step("Step"):
    ...         print ("Start step")
    """

    allured_testdir.parse_docstring_source()
    allured_testdir.run_with_allure("--capture={capture}".format(capture=capture))

    if_pytest_capture_ = is_not if capture == "no" else is_

    assert_that(allured_testdir.allure_report,
                has_property("attachments",
                             all_of(
                                 if_pytest_capture_(has_value(contains_string("Start fixture"))),
                                 if_pytest_capture_(has_value(contains_string("Stop fixture"))),
                                 if_pytest_capture_(has_value(contains_string("Start test"))),
                                 if_pytest_capture_(has_value(contains_string("Start step")))
                             )
                             )
                )


@pytest.mark.parametrize("capture", ["sys", "fd"])
def test_capture_empty_stdout(allured_testdir, capture):
    """
    >>> import pytest
    >>> import allure

    >>> @pytest.fixture
    ... def fixture(request):
    ...     def finalizer():
    ...         pass
    ...     request.addfinalizer(finalizer)

    >>> def test_capture_stdout_example(fixture):
    ...     with allure.step("Step"):
    ...         pass
    """

    allured_testdir.parse_docstring_source()
    allured_testdir.run_with_allure("--capture={capture}".format(capture=capture))

    assert_that(allured_testdir.allure_report,
                has_property("attachments", empty())
                )


@pytest.mark.parametrize("logging", [True, False])
def test_capture_log(allured_testdir, logging):
    """
    >>> import logging
    >>> import pytest
    >>> import allure

    >>> logger = logging.getLogger(__name__)

    >>> @pytest.fixture
    ... def fixture(request):
    ...     logger.info("Start fixture")
    ...     def finalizer():
    ...         logger.info("Stop fixture")
    ...     request.addfinalizer(finalizer)

    >>> def test_capture_log_example(fixture):
    ...     logger.info("Start test")
    ...     with allure.step("Step"):
    ...         logger.info("Start step")
    """

    allured_testdir.parse_docstring_source()

    params = [] if logging else ["-p", "no:logging"]
    if_logging_ = is_ if logging else is_not

    allured_testdir.run_with_allure("--log-cli-level=INFO", *params)

    assert_that(allured_testdir.allure_report,
                has_property("attachments",
                             all_of(
                                 if_logging_(has_value(contains_string("Start fixture"))),
                                 if_logging_(has_value(contains_string("Stop fixture"))),
                                 if_logging_(has_value(contains_string("Start test"))),
                                 if_logging_(has_value(contains_string("Start step")))
                             )
                             )
                )


def test_capture_disabled(allured_testdir):
    """
    >>> import logging
    >>> logger = logging.getLogger(__name__)

    >>> def test_capture_disabled_example():
    ...     logger.info("Start logging")
    ...     print ("Start printing")

    """

    allured_testdir.parse_docstring_source()
    allured_testdir.run_with_allure("--log-cli-level=INFO", "--allure-no-capture")

    assert_that(allured_testdir.allure_report,
                has_property("attachments", empty())
                )



""" ./examples/label/bdd/dynamic_bdd_label.rst """

import pytest
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_feature


def test_dynamic_feature(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_dynamic_feature",
                              has_feature("first feature"),
                              has_feature("second feature")
                              )
                )


@pytest.mark.parametrize("feature", ["first feature", "second feature"])
def test_parametrized_dynamic_feature(executed_docstring_path, feature):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_parametrized_dynamic_feature[{feature}]".format(feature=feature),
                              has_feature(feature),
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains
from allure_commons_test.container import has_container
from allure_commons_test.container import has_after


def test_xfail_failed_finalizer_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def failed_finalizer_fixture(request):
    ...     def fixture_finalizer():
    ...         assert False
    ...     request.addfinalizer(fixture_finalizer)

    >>> @pytest.mark.xfail()
    ... def test_xfail_failed_finalizer_fixture_example(failed_finalizer_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_xfail_failed_finalizer_fixture_example",
                              with_status("passed"),
                              has_status_details(with_message_contains("XPASS")),
                              has_container(executed_docstring_source.allure_report,
                                            has_after("{fixture}::{finalizer}".format(
                                                fixture="failed_finalizer_fixture",
                                                finalizer="fixture_finalizer"),
                                                with_status("failed"),
                                                has_status_details(with_message_contains("AssertionError"),
                                                                   with_trace_contains("fixture_finalizer")
                                                                   ),
                                            ),
                                            )
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_attachment


def test_attach_from_runtest_teardown(allured_testdir):
    allured_testdir.testdir.makeconftest("""
        import allure


        def pytest_runtest_teardown(*args, **kwargs):
            allure.attach(body="body", name="attachment from teardown")
    """)

    allured_testdir.testdir.makepyfile("""
        def test_attach_from_runtest_teardown():
            pass
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_attach_from_runtest_teardown",
                              has_attachment(name="attachment from teardown"),
                              )
                )


def test_attach_from_runtest_logfinish(allured_testdir):
    allured_testdir.testdir.makeconftest("""
        import allure


        def pytest_runtest_logfinish(*args, **kwargs):
            allure.attach(body="body", name="attachment from logfinish")
    """)

    allured_testdir.testdir.makepyfile("""
        def test_attach_from_runtest_logfinish():
            pass
    """)

    allured_testdir.run_with_allure()

    assert_that(allured_testdir.allure_report,
                has_test_case("test_attach_from_runtest_logfinish",
                              has_attachment(name="attachment from logfinish"),
                              )
                )



""" ./examples/link/link.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_link
from allure_commons_test.result import has_issue_link
from allure_commons_test.result import has_test_case_link


def test_link(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_link",
                              has_link("http://qameta.io")
                              )
                )


def test_issue_link(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_issue_link",
                              has_issue_link("https://github.com/allure-framework/allure-python/issues/24")
                              )
                )


def test_testcase_link(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_testcase_link",
                              has_test_case_link("issues/24#issuecomment-277330977")
                              )
                )


def test_custom_link(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_custom_link",
                              has_link("http://qameta.io", name="QAMETA", link_type="homepage")
                              )
                )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import pytest
import allure_commons
from allure_pytest.utils import ALLURE_DISPLAY_NAME_MARK
from allure_pytest.utils import ALLURE_DESCRIPTION_MARK, ALLURE_DESCRIPTION_HTML_MARK
from allure_pytest.utils import ALLURE_LABEL_MARK, ALLURE_LINK_MARK


class AllureTestHelper(object):

    def __init__(self, config):
        self.config = config

    @allure_commons.hookimpl
    def decorate_as_title(self, test_title):
        allure_title = getattr(pytest.mark, ALLURE_DISPLAY_NAME_MARK)
        return allure_title(test_title)

    @allure_commons.hookimpl
    def decorate_as_description(self, test_description):
        allure_description = getattr(pytest.mark, ALLURE_DESCRIPTION_MARK)
        return allure_description(test_description)

    @allure_commons.hookimpl
    def decorate_as_description_html(self, test_description_html):
        allure_description_html = getattr(pytest.mark, ALLURE_DESCRIPTION_HTML_MARK)
        return allure_description_html(test_description_html)

    @allure_commons.hookimpl
    def decorate_as_label(self, label_type, labels):
        allure_label = getattr(pytest.mark, ALLURE_LABEL_MARK)
        return allure_label(*labels, label_type=label_type)

    @allure_commons.hookimpl
    def decorate_as_link(self, url, link_type, name):
        pattern = dict(self.config.option.allure_link_pattern).get(link_type, u'{}')
        url = pattern.format(url)
        allure_link = getattr(pytest.mark, ALLURE_LINK_MARK)
        name = url if name is None else name
        return allure_link(url, name=name, link_type=link_type)



""" ./examples/step/step.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_step


def test_inline_step(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_inline_step",
                              has_step("inline step")
                              )
                )


def test_reusable_step(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_reusable_step",
                              has_step("passed_step")
                              )
                )


def test_nested_steps(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_nested_steps",
                              has_step("grand parent step",
                                       has_step("parent step",
                                                has_step("passed_step"
                                                         )
                                                )
                                       )
                              )
                )


def test_class_method_as_step(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_class_method_as_step",
                              has_step("class method as step")
                              )
                )



""" ./examples/label/bdd/bdd_label.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.label import has_epic
from allure_commons_test.label import has_feature
from allure_commons_test.label import has_story


def test_single_bdd_label(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_single_bdd_label",
                              has_epic("My epic"),
                              has_feature("My feature"),
                              has_story("My story")
                              )
                )


def test_multiple_bdd_label(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_multiple_bdd_label",
                              has_epic("My epic"),
                              has_epic("Another epic"),
                              has_feature("My feature"),
                              has_feature("Another feature"),
                              has_feature("One more feature"),
                              has_story("My story"),
                              has_story("Alternative story")
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains


def test_skip(executed_docstring_source):
    """
    >>> import pytest

    >>> def test_skip_example():
    ...     pytest.skip()
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_skip_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Skipped: <Skipped instance>"))
                              )
                )


def test_skip_with_reason(executed_docstring_source):
    """
    >>> import pytest

    >>> def test_skip_with_reason_example():
    ...     pytest.skip("Skip reason")
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_skip_with_reason_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Skipped: Skip reason"))
                              )
                )


def test_skip_decorator_and_reason(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.skip(reason="Skip reason")
    ... def test_skip_decorator_and_reason_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_skip_decorator_and_reason_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Skipped: Skip reason"))
                              )
                )


def test_skipif_true(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.skipif(True, reason="Skip reason")
    ... def test_skipif_true_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_skipif_true_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Skipped: Skip reason"))
                              )
                )


def test_skipif_false(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.mark.skipif(False, reason="Skip reason")
    ... def test_skipif_false_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_skipif_false_example",
                              with_status("passed")
                              )
                )



"""./examples/description/dynamic_description.rst"""

from hamcrest import assert_that, contains_string
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_description, has_description_html


def test_dynamic_description(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_dynamic_description",
                              has_description(contains_string("Actual description"))
                              )
                )


def test_dynamic_description_html(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_dynamic_description_html",
                              has_description_html(contains_string("<p>Actual HTML description</p>"))
                              )
                )



import allure
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import with_status
from allure_commons_test.result import has_status_details
from allure_commons_test.result import with_message_contains
from allure_commons_test.result import with_trace_contains
from allure_commons_test.container import has_container
from allure_commons_test.container import has_before


@allure.feature("Fixture")
def test_skip_fixture(executed_docstring_source):
    """
    >>> import pytest

    >>> @pytest.fixture
    ... def skip_fixture():
    ...     pytest.skip()

    >>> @pytest.mark.xfail()
    ... def test_skip_fixture_example(skip_fixture):
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_skip_fixture_example",
                              with_status("skipped"),
                              has_status_details(with_message_contains("Skipped: <Skipped instance>")),
                              has_container(executed_docstring_source.allure_report,
                                            has_before("skip_fixture",
                                                       with_status("skipped"),
                                                       has_status_details(
                                                           with_message_contains("Skipped: <Skipped instance>"),
                                                           with_trace_contains("skip_fixture")
                                                       ),
                                                       ),
                                            )
                              )
                )



import pytest
from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_parameter


def params_name(request):
    node_id = request.node.nodeid
    _, name = node_id.rstrip("]").split("[")
    return name


@pytest.mark.parametrize("param", [True, False])
def test_parametrization(executed_docstring_source, param):
    """
    >>> import pytest

    >>> @pytest.mark.parametrize("param", [True, False])
    ... def test_parametrization_example(param):
    ...     assert param
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_parametrization_example[{param}]".format(param=param),
                              has_parameter("param", str(param))
                              )
                )


@pytest.mark.xfail()
@pytest.mark.parametrize("param", [True, False], ids=["pass", "fail"])
def test_parametrization_with_ids(executed_docstring_source, param):
    """
    >>> import pytest

    >>> @pytest.mark.parametrize("param", [True, False], ids=["pass", "fail"])
    ... def test_parametrization_with_ids_example(param):
    ...     assert param
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case(
                    "test_parametrization_with_ids_example[{param}]".format(param="pass" if param else "fail"),
                    has_parameter("pass" if param else "fail", str(param))
                )
                )


@pytest.mark.parametrize("param1", [True, False])
@pytest.mark.parametrize("param2", [True, True])
def test_parametrization_many_decorators(executed_docstring_source, request, param1, param2):
    """
    >>> import pytest

    >>> @pytest.mark.parametrize("param1", [True, False])
    ... @pytest.mark.parametrize("param2", [True, True])
    ... def test_parametrization_many_decorators_example(param1, param2):
    ...     pass
    """

    test_name = "test_parametrization_many_decorators_example[{params_name}]".format(params_name=params_name(request))

    assert_that(executed_docstring_source.allure_report,
                has_test_case(test_name,
                              has_parameter("param1", str(param1)),
                              has_parameter("param2", str(param2))

                              )
                )


@pytest.mark.xfail()
@pytest.mark.parametrize("param1", [True, False], ids=["pass", "fail"])
@pytest.mark.parametrize("param2", [True, True])
def test_parametrization_many_decorators_with_partial_ids(executed_docstring_source, request, param1, param2):
    """
    >>> import pytest

    >>> @pytest.mark.parametrize("param1", [True, False], ids=["first_pass", "first_fail"])
    ... @pytest.mark.parametrize("param2", [True, True])
    ... def test_parametrization_many_decorators_with_partial_ids_example(param1, param2):
    ...     pass
    """

    test_name = "test_parametrization_many_decorators_with_partial_ids_example[{params_name}]".format(
        params_name=params_name(request))

    assert_that(executed_docstring_source.allure_report,
                has_test_case(test_name,
                              has_parameter("pass" if param1 else "fail", str(param1)),
                              has_parameter("param2", str(param2))

                              )
                )



""" ./examples/attachment/attachment_step.rst """

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_step
from allure_commons_test.result import has_attachment


def test_step_with_attachment(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_step_with_attachment",
                              has_step("step_with_attachment",
                                       has_attachment()
                                       ),
                              )
                )



import pytest
import six
from attr import asdict
from allure_commons import hookimpl
from allure_commons_test.report import AllureReport
from doctest import script_from_examples
import mock
import allure_commons
from contextlib import contextmanager


class AllureMemoryLogger(object):
    def __init__(self):
        self.test_cases = []
        self.test_containers = []
        self.attachments = {}

    @hookimpl
    def report_result(self, result):
        data = asdict(result, filter=lambda attr, value: not (type(value) != bool and not bool(value)))
        self.test_cases.append(data)

    @hookimpl
    def report_container(self, container):
        data = asdict(container, filter=lambda attr, value: not (type(value) != bool and not bool(value)))
        self.test_containers.append(data)

    @hookimpl
    def report_attached_file(self, source, file_name):
        pass

    @hookimpl
    def report_attached_data(self, body, file_name):
        self.attachments[file_name] = body


@contextmanager
def fake_logger(path, logger):
    blocked_plugins = []
    for name, plugin in allure_commons.plugin_manager.list_name_plugin():
        allure_commons.plugin_manager.unregister(plugin=plugin, name=name)
        blocked_plugins.append(plugin)

    with mock.patch(path) as ReporterMock:
        ReporterMock.return_value = logger
        yield

    for plugin in blocked_plugins:
        allure_commons.plugin_manager.register(plugin)


class AlluredTestdir(object):
    def __init__(self, testdir, request):
        self.testdir = testdir
        self.request = request
        self.allure_report = None

    def parse_docstring_source(self):
        docstring = self.request.node.function.__doc__ or self.request.node.module.__doc__
        source = script_from_examples(docstring).replace("#\n", "\n")
        if six.PY2:
            self.testdir.makepyfile("# -*- coding: utf-8 -*-\n%s" % source)
        else:
            self.testdir.makepyfile(source)

    def parse_docstring_path(self):
        doc_file = self.request.node.function.__doc__ or self.request.node.module.__doc__
        example_dir = self.request.config.rootdir.join(doc_file.strip())

        if six.PY2:
            with open(str(example_dir)) as f:
                content = "# -*- coding: utf-8 -*-\n%s" % f.read()
                source = script_from_examples(content)
                self.testdir.makepyfile(source)
        else:
            with open(example_dir, encoding="utf-8") as f:

                content = f.read()
                source = script_from_examples(content)
                self.testdir.makepyfile(source)

    def run_with_allure(self, *args, **kwargs):
        if self.request.node.get_closest_marker("real_logger"):
            self.testdir.runpytest("--alluredir", self.testdir.tmpdir, *args, **kwargs)
            self.allure_report = AllureReport(self.testdir.tmpdir.strpath)
        else:
            self.allure_report = AllureMemoryLogger()
            with fake_logger("allure_pytest.plugin.AllureFileLogger", self.allure_report):
                self.testdir.runpytest("--alluredir", self.testdir.tmpdir, *args, **kwargs)

        return self.allure_report


@pytest.fixture
def allured_testdir(testdir, request):
    return AlluredTestdir(testdir, request)


@pytest.fixture
def executed_docstring_source(allured_testdir):
    allured_testdir.parse_docstring_source()
    allured_testdir.run_with_allure()
    return allured_testdir


@pytest.fixture
def executed_docstring_path(allured_testdir):
    allured_testdir.parse_docstring_path()
    allured_testdir.run_with_allure()
    return allured_testdir



# -*- coding: utf-8 -*-
""" ./examples/display_name/display_name.rst"""

from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_title


def test_display_name(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_display_name",
                              has_title("A some test title")
                              )
                )


def test_display_name_template(executed_docstring_path):
    assert_that(executed_docstring_path.allure_report,
                has_test_case("test_display_name_template",
                              has_title("A some test title with param False")
                              )
                )


def test_unicode_display_name(executed_docstring_source):
    """
    >>> import allure

    >>> @allure.title(u"Лунтик")
    >>> def test_unicode_display_name_example():
    ...     pass
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_unicode_display_name_example", has_title(u"Лунтик"))
                )


def test_unicode_display_name_template(executed_docstring_source):
    """
    >>> import allure
    >>> import pytest

    >>> @allure.title(u"Тест с шаблоном и параметром: {param}")
    ... @pytest.mark.parametrize("param", [False])
    ... def test_unicode_display_name_template_example(param):
    ...     assert param
    """

    assert_that(executed_docstring_source.allure_report,
                has_test_case("test_unicode_display_name_template_example",
                              has_title(u"Тест с шаблоном и параметром: False")
                              )
                )



from hamcrest import assert_that
from allure_commons_test.report import has_test_case
from allure_commons_test.result import has_link, has_issue_link


def test_link_pattern(allured_testdir):
    """ ./examples/link/dynamic_link.rst """

    allured_testdir.parse_docstring_path()

    allured_testdir.run_with_allure("--allure-link-pattern",
                                    "issue:https://github.com/allure-framework/allure-python2/{}",
                                    "--allure-link-pattern",
                                    "docs:https://docs.qameta.io/{}")

    assert_that(allured_testdir.allure_report,
                has_test_case("test_all_links_together",
                              has_issue_link("https://github.com/allure-framework/allure-python2/issues/24"),
                              has_issue_link("https://github.com/allure-framework/allure-python2/issues/24"),
                              has_link("https://docs.qameta.io/allure")
                              )
                )



import pytest
from hamcrest import assert_that
from hamcrest import all_of
from hamcrest import has_property, has_value
from hamcrest import contains_string
from allure_commons_test.report import has_test_case


@pytest.mark.parametrize("param", ["first", "second"])
def test_parametrized_attachment(executed_docstring_source, param):
    """
    >>> import pytest
    >>> import allure

    >>> @pytest.mark.parametrize("param", ["first", "second"])
    ... def test_parametrized_attachment_example(param):
    ...     allure.attach(param)
    """

    assert_that(executed_docstring_source.allure_report,
                all_of(
                    has_test_case("test_parametrized_attachment_example[{param}]".format(param=param)),
                    has_property("attachments",
                                 has_value(contains_string(param))
                                 )
                ))
